@@ -27,10 +27,8 @@
 #include "sox_i.h"
 #include "fft4g.h"
 #include "dft_filter.h"
-#include <assert.h>
-#include <string.h>
 
-#define raw_coef_t double
+typedef double raw_coef_t;
 
 #if 0 /* For float32 version, as used in foobar */
   #define sample_t   float
@@ -54,9 +52,11 @@ static sample_t * prepare_coefs(raw_coef_t const * coefs, int num_coefs,
     int num_phases, int interp_order, int multiplier)
 {
   int i, j, length = num_coefs4 * num_phases;
-  sample_t * result = lsx_malloc(length * (interp_order + 1) * sizeof(*result));
+  sample_t * result;
   double fm1 = coefs[0], f1 = 0, f2 = 0;
 
+  lsx_valloc(result, length * (interp_order + 1));
+
   for (i = num_coefs4 - 1; i >= 0; --i)
     for (j = num_phases - 1; j >= 0; --j) {
       double f0 = fm1, b = 0, c = 0, d = 0; /* = 0 to kill compiler warning */
@@ -233,7 +233,7 @@ static int dft_stage_init(
       return SOX_EINVAL;
     }
 
-    f->coefs = lsx_calloc(dft_length, sizeof(*f->coefs));
+    lsx_vcalloc(f->coefs, dft_length);
     for (i = 0; i < num_taps; ++i)
       f->coefs[(i + dft_length - num_taps + 1) & (dft_length - 1)]
         = h[i] / dft_length * 2 * L;
@@ -349,7 +349,7 @@ static int rate_init(
   if (!p->num_stages)
     return SOX_SUCCESS;
 
-  p->stages = lsx_calloc(p->num_stages + 1, sizeof(*p->stages));
+  lsx_vcalloc(p->stages, p->num_stages + 1);
   for (i = 0; i < p->num_stages; ++i)
     p->stages[i].shared = shared;
 
@@ -501,7 +501,9 @@ static void rate_flush(rate_t * p)
   uint64_t samples_out = p->samples_in / p->factor + .5;
   size_t remaining = samples_out > p->samples_out ?
       (size_t)(samples_out - p->samples_out) : 0;
-  sample_t * buff = lsx_calloc(1024, sizeof(*buff));
+  sample_t * buff;
+
+  lsx_vcalloc(buff, 1024);
 
   if (remaining > 0) {
     while ((size_t)fifo_occupancy(fifo) < remaining) {
@@ -707,27 +709,30 @@ static int stop(sox_effect_t * effp)
 
 sox_effect_handler_t const * lsx_rate_effect_fn(void)
 {
+  static const char usage[] =
+    "[-q|-l|-m|-h|-v] [override-options] rate[k]";
+
+  static char const * const extra_usage[] = {
+"    QUALITY    BANDWIDTH  REJ dB   TYPICAL USE",
+"-q  quick          n/a  ~30 @ Fs/4 playback on ancient hardware",
+"-l  low            80%     100     playback on old hardware",
+"-m  medium         95%     100     audio playback",
+"-h  high (default) 95%     125     16-bit mastering (use with dither)",
+"-v  very high      95%     175     24-bit mastering",
+"OVERRIDE OPTIONS (only with -m, -h, -v)",
+"-M/-I/-L     Phase response = minimum/intermediate/linear(default)",
+"-s           Steep filter (band-width = 99%)",
+"-a           Allow aliasing above the pass-band",
+"-b 74-99.7   Any band-width %",
+"-p 0-100     Any phase response (0 = minimum, 25 = intermediate,",
+"             50 = linear, 100 = maximum)",
+    NULL
+  };
+
   static sox_effect_handler_t handler = {
-    "rate", 0, SOX_EFF_RATE, create, start, flow, drain, stop, 0, sizeof(priv_t)
+    "rate", usage, extra_usage, SOX_EFF_RATE,
+    create, start, flow, drain, stop, 0, sizeof(priv_t)
   };
-  static char const * lines[] = {
-    "[-q|-l|-m|-h|-v] [override-options] RATE[k]",
-    "                    BAND-",
-    "     QUALITY        WIDTH  REJ dB   TYPICAL USE",
-    " -q  quick          n/a  ~30 @ Fs/4 playback on ancient hardware",
-    " -l  low            80%     100     playback on old hardware",
-    " -m  medium         95%     100     audio playback",
-    " -h  high (default) 95%     125     16-bit mastering (use with dither)",
-    " -v  very high      95%     175     24-bit mastering",
-    "              OVERRIDE OPTIONS (only with -m, -h, -v)",
-    " -M/-I/-L     Phase response = minimum/intermediate/linear(default)",
-    " -s           Steep filter (band-width = 99%)",
-    " -a           Allow aliasing above the pass-band",
-    " -b 74-99.7   Any band-width %",
-    " -p 0-100     Any phase response (0 = minimum, 25 = intermediate,",
-    "              50 = linear, 100 = maximum)",
-  };
-  static char * usage;
-  handler.usage = lsx_usage_lines(&usage, lines, array_length(lines));
+
   return &handler;
 }
