@@ -22,9 +22,6 @@
 
 #ifdef HAVE_SNDFILE
 
-#include <assert.h>
-#include <stdio.h>
-#include <string.h>
 #include <ctype.h>
 #include <sndfile.h>
 
@@ -56,11 +53,9 @@ static const char* const sndfile_library_names[] =
 #endif
 #endif /* DL_SNDFILE */
 
-#define SNDFILE_FUNC_OPEN(f,x) \
-  SNDFILE_FUNC(f,x, SNDFILE*, sf_open_virtual, (SF_VIRTUAL_IO *sfvirtual, int mode, SF_INFO *sfinfo, void *user_data))
-
 #define SNDFILE_FUNC_ENTRIES(f,x) \
-  SNDFILE_FUNC_OPEN(f,x) \
+  SNDFILE_FUNC(f,x, SNDFILE*, sf_open, (const char *path, int mode, SF_INFO *sfinfo)) \
+  SNDFILE_FUNC(f,x, SNDFILE*, sf_open_virtual, (SF_VIRTUAL_IO *sfvirtual, int mode, SF_INFO *sfinfo, void *user_data)) \
   SNDFILE_FUNC_STOP(f,x, int, sf_stop, (SNDFILE *sndfile)) \
   SNDFILE_FUNC(f,x, int, sf_close, (SNDFILE *sndfile)) \
   SNDFILE_FUNC(f,x, int, sf_format_check, (const SF_INFO *info)) \
@@ -133,11 +128,8 @@ static int ft_enc(unsigned size, sox_encoding_t e)
   if (e == SOX_ENCODING_FLAC      && size == 16) return SF_FORMAT_PCM_16;
   if (e == SOX_ENCODING_FLAC      && size == 24) return SF_FORMAT_PCM_24;
   if (e == SOX_ENCODING_FLAC      && size == 32) return SF_FORMAT_PCM_32;
-#ifdef HAVE_SF_FORMAT_OGG
-  if (e == SOX_ENCODING_VORBIS)   return SF_FORMAT_VORBIS;
-#endif
 #ifdef HAVE_SF_FORMAT_MPEG
-  if (e == SOX_ENCODING_MP3)      return SF_FORMAT_MPEG_LAYER_III;
+  if (e == SOX_ENCODING_MP3       && size == 16) return SF_FORMAT_MPEG_LAYER_III;
 #endif
   return 0; /* Bad encoding */
 }
@@ -176,12 +168,11 @@ static sox_encoding_t sox_enc(int ft_encoding, unsigned * size)
     case SF_FORMAT_DWVW_24  : *size = 24; return SOX_ENCODING_DWVW;
     case SF_FORMAT_DWVW_N   : *size =  0; return SOX_ENCODING_DWVWN;
     case SF_FORMAT_GSM610   : *size =  0; return SOX_ENCODING_GSM;
-#if HAVE_SF_FORMAT_OGG
-    case SF_FORMAT_VORBIS   : *size =  0; return SOX_ENCODING_VORBIS;
-#endif
-#if HAVE_SF_FORMAT_MPEG
+#ifdef SF_FORMAT_MPEG_LAYER_II
     case SF_FORMAT_MPEG_LAYER_II
                             : *size = 16; return SOX_ENCODING_MP3;
+#endif
+#ifdef SF_FORMAT_MPEG_LAYER_III
     case SF_FORMAT_MPEG_LAYER_III
                             : *size = 16; return SOX_ENCODING_MP3;
 #endif
@@ -203,13 +194,13 @@ static struct {
   { "flac",     SF_FORMAT_FLAC },
   { "wve",      SF_FORMAT_WVE },  /* Probably broken before 1.0.18 */
 #ifdef HAVE_SF_FORMAT_OGG
-  { "ogg",      SF_FORMAT_OGG | SF_FORMAT_VORBIS },  /* From 1.0.16 */
+  { "ogg",      SF_FORMAT_OGG },  /* From 1.0.16 */
 #endif
 #ifdef HAVE_SF_FORMAT_MPC2K
   { "mpc2k",    SF_FORMAT_MPC2K },  /* From 1.0.25 */
 #endif
 #ifdef HAVE_SF_FORMAT_MPEG
-  { "mp3",      SF_FORMAT_MPEG | SF_FORMAT_MPEG_LAYER_III },  /* From 1.1.0 */
+  { "mp3",      SF_FORMAT_MPEG },  /* From 1.1.0 */
 #endif
   { "svx",      SF_FORMAT_SVX },
   { "8svx",     SF_FORMAT_SVX },
@@ -291,7 +282,7 @@ static int name_to_format(const char *name)
     buffer[FILE_TYPE_BUFLEN] = '\0';
 
     for (k = 0; buffer[k]; k++)
-      buffer[k] = tolower((int)buffer[k]);
+      buffer[k] = tolower((int)(buffer[k]));
   } else {
     strncpy(buffer, name, FILE_TYPE_BUFLEN);
     buffer[FILE_TYPE_BUFLEN] = '\0';
@@ -327,22 +318,9 @@ static int start(sox_format_t * ft)
   /* Copy format info */
   if (subtype) {
     if (strcmp(ft->filetype, "sndfile") == 0)
-      sf->sf_info->format = name_to_format(ft->filename);
+      sf->sf_info->format = name_to_format(ft->filename) | subtype;
     else
-      sf->sf_info->format = name_to_format(ft->filetype);
-
-    /* For linear types, name_to_format only returns the major format type
-     * and we need to OR in the bit depth */
-    switch (sf->sf_info->format) {
-    case SF_FORMAT_OGG | SF_FORMAT_VORBIS:
-#if HAVE_SF_FORMAT_MPEG
-    case SF_FORMAT_MPEG | SF_FORMAT_MPEG_LAYER_III:
-#endif
-      break;
-    default:
-      sf->sf_info->format |= subtype;
-      break;
-    }
+      sf->sf_info->format = name_to_format(ft->filetype) | subtype;
   }
   sf->sf_info->samplerate = (int)ft->signal.rate;
   sf->sf_info->channels = ft->signal.channels;
@@ -352,6 +330,21 @@ static int start(sox_format_t * ft)
   return SOX_SUCCESS;
 }
 
+/*
+ * Close file for libsndfile (this doesn't close the file handle)
+ */
+static int stop(sox_format_t * ft)
+{
+  priv_t * sf = (priv_t *)ft->priv;
+  sf->sf_stop(sf->sf_file);
+  drain_log_buffer(ft);
+  sf->sf_close(sf->sf_file);
+  free((void *)sf->log_buffer);
+  free((void *)sf->sf_info);
+  LSX_DLLIBRARY_CLOSE(sf, sndfile_dl);
+  return SOX_SUCCESS;
+}
+
 static int check_read_params(sox_format_t * ft, unsigned channels,
     sox_rate_t rate, sox_encoding_t encoding, unsigned bits_per_sample, uint64_t length)
 {
@@ -388,12 +381,23 @@ static int startread(sox_format_t * ft)
   unsigned bits_per_sample;
   sox_encoding_t encoding;
   sox_rate_t rate;
+  const char *extension = lsx_find_file_extension(ft->filename);
 
   if (start(ft) == SOX_EOF)
       return SOX_EOF;
 
-  sf->sf_file = sf->sf_open_virtual(&vio, SFM_READ, sf->sf_info, ft);
-  drain_log_buffer(ft);
+  /*
+   * The SD2 format relies on a resource fork,
+   * which cannot be supported with virtual IO.
+   */
+  if (extension && strcasecmp(extension, "sd2") == 0) {
+    memset(sf->sf_info, 0, sizeof (*sf->sf_info));
+    sf->sf_file = sf->sf_open(ft->filename, SFM_READ, sf->sf_info);
+    drain_log_buffer(ft);
+  } else {
+    sf->sf_file = sf->sf_open_virtual(&vio, SFM_READ, sf->sf_info, ft);
+    drain_log_buffer(ft);
+  }
 
   if (sf->sf_file == NULL) {
     memset(ft->sox_errstr, 0, sizeof(ft->sox_errstr));
@@ -403,7 +407,7 @@ static int startread(sox_format_t * ft)
   }
 
   if (!(encoding = sox_enc(sf->sf_info->format, &bits_per_sample))) {
-    lsx_fail_errno(ft, SOX_EFMT, "unsupported sndfile encoding %#x", sf->sf_info->format);
+    lsx_fail_errno(ft, SOX_EFMT, "unsupported encoding %#x", sf->sf_info->format);
     return SOX_EOF;
   }
 
@@ -438,19 +442,6 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
   return (size_t)sf->sf_read_int(sf->sf_file, (int *)buf, (sf_count_t)len);
 }
 
-/*
- * Close file for libsndfile (this doesn't close the file handle)
- */
-static int stopread(sox_format_t * ft)
-{
-  priv_t * sf = (priv_t *)ft->priv;
-  sf->sf_stop(sf->sf_file);
-  drain_log_buffer(ft);
-  sf->sf_close(sf->sf_file);
-  LSX_DLLIBRARY_CLOSE(sf, sndfile_dl);
-  return SOX_SUCCESS;
-}
-
 static int startwrite(sox_format_t * ft)
 {
   priv_t * sf = (priv_t *)ft->priv;
@@ -480,14 +471,22 @@ static int startwrite(sox_format_t * ft)
       return SOX_EOF;
     }
     if ((sf->sf_info->format & SF_FORMAT_TYPEMASK) != SF_FORMAT_RAW 
-#if HAVE_SF_FORMAT_MPEG
+#ifdef SF_FORMAT_MPEG
         && (sf->sf_info->format & SF_FORMAT_TYPEMASK) != SF_FORMAT_MPEG
 #endif
 	)
       lsx_warn("cannot use desired output encoding, choosing default");
   }
 
-  sf->sf_file = sf->sf_open_virtual(&vio, SFM_WRITE, sf->sf_info, ft);
+  /*
+   * The SD2 format relies on a resource fork,
+   * which cannot be supported with virtual IO.
+   */
+  if ((sf->sf_info->format & SF_FORMAT_TYPEMASK) == SF_FORMAT_SD2) {
+    sf->sf_file = sf->sf_open(ft->filename, SFM_WRITE, sf->sf_info);
+  } else {
+    sf->sf_file = sf->sf_open_virtual(&vio, SFM_WRITE, sf->sf_info, ft);
+  }
   drain_log_buffer(ft);
 
   if (sf->sf_file == NULL) {
@@ -516,19 +515,6 @@ static size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, size_t l
   return (size_t)sf->sf_write_int(sf->sf_file, (int *)buf, (sf_count_t)len);
 }
 
-/*
- * Close file for libsndfile (this doesn't close the file handle)
- */
-static int stopwrite(sox_format_t * ft)
-{
-  priv_t * sf = (priv_t *)ft->priv;
-  sf->sf_stop(sf->sf_file);
-  drain_log_buffer(ft);
-  sf->sf_close(sf->sf_file);
-  LSX_DLLIBRARY_CLOSE(sf, sndfile_dl);
-  return SOX_SUCCESS;
-}
-
 static int seek(sox_format_t * ft, sox_uint64_t offset)
 {
   priv_t * sf = (priv_t *)ft->priv;
@@ -544,6 +530,7 @@ LSX_FORMAT_HANDLER(sndfile)
     /* "aif", */
     /* "au", */
     /* "gsm", */
+    /* "mp3", */
     /* "nist", */
     /* "raw", */
     /* "sf", "ircam", */
@@ -552,10 +539,9 @@ LSX_FORMAT_HANDLER(sndfile)
     /* "voc", */
     /* "vox", */
     /* "wav", */
+    /* "wve", */
   /* LSF wrappers of formats already wrapped in SoX: */
     /* "flac", */
-
-    "sds",  /* ?? */
     NULL
   };
 
@@ -569,12 +555,13 @@ LSX_FORMAT_HANDLER(sndfile)
     SOX_ENCODING_MS_ADPCM, 4, 0,
     SOX_ENCODING_OKI_ADPCM, 4, 0,
     SOX_ENCODING_GSM, 0,
+    SOX_ENCODING_MP3, 0,
     0};
 
   static sox_format_handler_t const format = {SOX_LIB_VERSION_CODE,
     "Pseudo format to use libsndfile", names, 0,
-    startread, read_samples, stopread,
-    startwrite, write_samples, stopwrite,
+    startread, read_samples, stop,
+    startwrite, write_samples, stop,
     seek, write_encodings, NULL, sizeof(priv_t)
   };
 
